%{
/*******************************************/
/*                                         */
/*  File: chtml.l                          */
/*  Purpose: c2html lex definition         */
/*                                         */
/*  Author: Sfiligoi Igor                  */
/*                                         */
/*  Created      : 30.01.1997              */
/*  Last modified: 25.03.1997              */
/*                                         */
/*******************************************/

#include <stdlib.h>
#include <string.h>

/* max token size */
#undef YYLMAX
#define YYLMAX	64*1024

/* set eventual error reporter */
#undef output
#define output(c) fprintf(stderr,"Lex line %i: char %c\n",Curr_Line,c);

#include "chtmlprocs.h"

int yywrap();
void lex_string_err( void );
%}


uletter		[_A-Za-z]
digit		[0-9]
uletterdigit	({uletter}|{digit})

identifier	({uletter}{uletterdigit}*)

blank		(" "|\t)
newline		\n

blankend	(\\{newline})

instring1	([^\"\n]|\\\")
string1		(\"{instring1}*\")

instring2	([^\'\n]|\\\')
string2		(\'{instring2}*\')

string		({string1}|{string2})

comment		("/*"([^*]*"*"+[^*/])*[^*]*"*"+"/")

space		(({blank}|{comment}|{newline})*)
spacennl	(({blank}|{blankend}|{comment})*)

precompiler	("#"{identifier})

includestr	("<"[^">"]*">")

special		("{"|"}"|"("|")"|"["|"]"|";"|"*"|","|"=")

struni		("struct"|"union"|"enum")

typeidf		("const"|"volatile"|"signed"|"unsigned"|"auto"|"register"|"static"|"extern")

other		[^\[\]\(\)\{\};\*,=\"\'\n]

%%


{space}			{ /* can be present only at the very begining of the file */
			 if (atbegin)
				{
				 yylval.str = str2htmlmem((char *) yytext);
				 atbegin=0;
				 return l_SPACE;
				}
			 else
				REJECT;
			}

{includestr}{spacennl}	
			{/* This is a string ONLY during an include */
			 if (ininclude) 
				{
				 str2htmlmem_string((char *) yytext,&(yylval.strrec));
				 return l_STRING;
				}
			 else
				REJECT;		/* Continue parsing */
			}

"#define"{spacennl}	{
			 yylval.str = str2htmlmem((char *) yytext);
			 inprecomp=1;
			 return l_DEFINE;
			}
"#include"{spacennl}	{
			 yylval.str = str2htmlmem((char *) yytext);
			 inprecomp=1;
			 ininclude=1;
			 return l_INCLUDE;
			}

{precompiler}{spacennl}	{
			 yylval.str = str2htmlmem((char *) yytext);
			 inprecomp=1;
			 return l_PRECOMPILER;
			}

"typedef"{space}	{ /* wrong space for inprecomp */
			 if (!inprecomp)
				{
				 yylval.str = str2htmlmem((char *) yytext);
				 return l_TYPEDEF;
				}
			 else
				REJECT;
			}
"typedef"{spacennl}	{ /* wrong space outside inprecomp */
			 if (inprecomp)
				{
				 yylval.str = str2htmlmem((char *) yytext);
				 return l_TYPEDEF;
				}
			 else
				REJECT;
			}

{struni}{space}		{ /* wrong space for inprecomp */
			 if (!inprecomp)
				{
				 yylval.str = str2htmlmem((char *) yytext);
				 return l_STRUNI;
				}
			 else
				REJECT;
			}

{struni}{spacennl}	{ /* wrong space outside inprecomp */
			 if (inprecomp)
				{
				 yylval.str = str2htmlmem((char *) yytext);
				 return l_STRUNI;
				}
			 else
				REJECT;
			}

{typeidf}{space}	{ /* wrong space for inprecomp */
			 if (!inprecomp)
				{
				 yylval.str = str2htmlmem((char *) yytext);
				 return l_TYPEIDF;
				}
			 else
				REJECT;
			}

{typeidf}{spacennl}	{ /* wrong space outside inprecomp */
			 if (inprecomp)
				{
				 yylval.str = str2htmlmem((char *) yytext);
				 return l_TYPEIDF;
				}
			 else
				REJECT;
			}

{identifier}{space}	{ /* wrong space for inprecomp */
			 if (!inprecomp)
				{
				 char *tmp;

				 extractidf((char *) yytext,&(yylval.idfstr.idf),&(tmp));
				 if (doprint)
					 fprintf(stderr,"%s'",yylval.idfstr.idf);
				 yylval.idfstr.succ = str2htmlmem(tmp);
				 free(tmp);
				 return l_IDENTIFIER;
				}
			 else
				REJECT;
			}
{identifier}{spacennl}	{ /* wrong space outside inprecomp */
			 if (inprecomp)
				{
				 char *tmp;

				 extractidf((char *) yytext,&(yylval.idfstr.idf),&(tmp));
				 if (doprint)
					 fprintf(stderr,"%s'",yylval.idfstr.idf);
				 yylval.idfstr.succ = str2htmlmem(tmp);
				 free(tmp);
				 return l_IDENTIFIER;
				}
			 else
				REJECT;
			}

{identifier}"("{spacennl}	{ /* wrong outside inprecomp */
				 if (inprecomp)
					{
					 char *tmp;
	
					 extractidf((char *) yytext,&(yylval.idfstr.idf),&(tmp));
					 if (doprint)
						 fprintf(stderr,"%s'",yylval.idfstr.idf);
					 yylval.idfstr.succ = str2htmlmem(tmp);
					 free(tmp);
					 return l_FUNC;
					}
				 else
					REJECT;
				}

{string}{space}		{ /* wrong space for inprecomp */
			 if (!inprecomp)
				{
				 str2htmlmem_string((char *) yytext, &(yylval.strrec));
				 return l_STRING;
				}
			 else
				REJECT;
			}
{string}{spacennl}	{ /* wrong space outside inprecomp */
			 if (inprecomp)
				{
				 str2htmlmem_string((char *) yytext, &(yylval.strrec));
				 return l_STRING;
				}
			 else
				REJECT;
			}

{newline}{space}	{ /* This is a newline only if IN precompiler */
			 if (inprecomp)
				{
				 yylval.str = str2htmlmem((char *) yytext);
				 inprecomp=0;
				 ininclude=0;  /* include is also a precomp */
				 return l_NEWLINE;
				}
			 else
				REJECT;
			}

"..."{space}		{ /* wrong space for inprecomp */
			 if (!inprecomp)
				{
				 yylval.str = str2htmlmem((char *) yytext);
				 return l_TPOINT;
				}
			 else
				REJECT;
			}

"..."{spacennl}		{ /* wrong space outside inprecomp */
			 if (inprecomp)
				{
				 yylval.str = str2htmlmem((char *) yytext);
				 return l_TPOINT;
				}
			 else
				REJECT;
			}

{special}{space}	{ /* wrong space for inprecomp */
			 if (!inprecomp)
				{
				 yylval.str = str2htmlmem((char *) yytext);
				 return yytext[0];
				}
			 else
				REJECT;
			}

{special}{spacennl}	{ /* wrong space outside inprecomp */
			 if (inprecomp)
				{
				 yylval.str = str2htmlmem((char *) yytext);
				 return yytext[0];
				}
			 else
				REJECT;
			}

{other}{space}		{ /* wrong space for inprecomp */
			 if (!inprecomp)
				{
				 yylval.str = str2htmlmem((char *) yytext);
				 return l_OTHER;
				}
			 else
				REJECT;
			}

{other}{spacennl}	{ /* wrong space outside inprecomp */
			 if (inprecomp)
				{
				 yylval.str = str2htmlmem((char *) yytext);
				 return l_OTHER;
				}
			 else
				REJECT;
			}

%%

/* report lex error in string */
void lex_string_err( void )
{
 fprintf(stderr,"lex line %8i: Error in string.\n",Curr_Line);
}

/* standard lex procedure */
int yywrap()
{
 return 1;
}

#include "chtmlprocs.c"

