%{
/*******************************************/
/*                                         */
/*  File: chtml.y                          */
/*  Purpose: c2html yacc definition        */
/*                                         */
/*  Author: Sfiligoi Igor                  */
/*                                         */
/*  Created      : 30.01.1997              */
/*  Last modified: 26.03.1997              */
/*                                         */
/*******************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "chtmlprocs.h"
#include "chtmldefs.h"
#include "chtmlidfs.h"

%}

%union	{
	 char *str;		/* general puropse string */
	 IDFSTR idfstr;		/* identifier + string */
	 STRREC strrec;         /* string */
	}

%token l_IDENTIFIER		/* type: idfstr */
%token l_FUNC			/* type: idfstr, similar to l_IDENTIFIER, but followed by a (, only present in precomp */
%token l_SPACE			/* type: str , can be present only at the begining*/
%token l_NEWLINE		/* type: str , can be present only in a precompiler*/
%token l_STRING			/* type: strrec */
%token l_TPOINT			/* type: str */
%token l_OTHER			/* type: str */

%token l_TYPEDEF		/* type: str */
%token l_STRUNI			/* type: str (struct, union and enum)*/
%token l_TYPEIDF		/* type: str */

%token l_DEFINE			/* type: str */
%token l_INCLUDE		/* type: str */
%token l_PRECOMPILER		/* type: str */

%start program

%%
/************************************************************************/

program		:	space elements
			{ /* concatena le strighe*/
			 strcatmem(finalprog,$1.str,$2.str);
			 /* non ritorna niente */
			}
		;

/* space including the newline and comments, or empty*/

space		:	l_SPACE
		|	{ /* empty */
			 atbegin=0;
			 stremptymem($$.str);
			}
		;	/* type: str */

	/* everything but l_NEWLINE */
nnlel		:	l_IDENTIFIER
			{ /* concatena le strighe */
			  char *elurl;
			  if ((elurl=hash_find(mainSTable,$1.idfstr.idf))!=NULL)
			    { /* link */
			      $$.str = (char *) malloc(strlen($1.idfstr.idf)+strlen($1.idfstr.succ)+
						       strlen(elurl)+18);
			      sprintf($$.str,"<A HREF=\"%s\">%s</A>%s",elurl,$1.idfstr.idf,$1.idfstr.succ);
			      free($1.idfstr.idf);
			      free($1.idfstr.succ);
			    }
			  else
			    {
			      strcatmem($$.str,$1.idfstr.idf,$1.idfstr.succ);
			    }
			}
		|	l_FUNC
			{ /* concatena le strighe */
			  char *elurl;
			  if ((elurl=hash_find(mainSTable,$1.idfstr.idf))!=NULL)
			    { /* link */
			      $$.str = (char *) malloc(strlen($1.idfstr.idf)+strlen($1.idfstr.succ)+
						       strlen(elurl)+18);
			      sprintf($$.str,"<A HREF=\"%s\">%s</A>%s",elurl,$1.idfstr.idf,$1.idfstr.succ);
			      free($1.idfstr.idf);
			      free($1.idfstr.succ);
			    }
			  else
			    {
			      strcatmem($$.str,$1.idfstr.idf,$1.idfstr.succ);
			    }
			}
		|	l_STRING
			{
			  strcat3mem($$.str,$1.strrec.pred,$1.strrec.mid,$1.strrec.succ);
			  free($1.strrec.name);
			}
		|	l_TPOINT
		|	l_OTHER
		|	l_TYPEDEF
		|	l_STRUNI
		|	l_TYPEIDF
		|	l_DEFINE
		|	l_INCLUDE
		|	l_PRECOMPILER
		|	'{'
		|	'}'
		|	'('
		|	')'
		|	'['
		|	']'
		|	'*'
		|	';'
		|	','
		|	'='
		;	/* type: str */

/* idfplus: identiefier or identifier+array*/
idfplus		:	l_IDENTIFIER
		|	l_IDENTIFIER arrays
			{ /* concatena le strighe */
			 strcatmem($$.idfstr.succ,$1.idfstr.succ,$2.str);
			 $$.idfstr.idf = $1.idfstr.idf;
			}
		;	/* type: idfstr */

arrays		:	arrays array
			{ /* concatena le strighe */
			 strcatmem($$.str,$1.str,$2.str);
			}
		|	array
		;	/* type: str */

array		:	'[' arrayels ']'
			{ /* concatena le strighe */
			 strcat3mem($$.str,$1.str,$2.str,$3.str);
			}
		;	/* type: str */


arrayels	:	arrayelsne
		|	{ /* empty */
			 stremptymem($$.str);
			}
		;	/* type: str */

arrayelsne	:	arrayelsne arrayel
			{ /* concatena le strighe */
			 strcatmem($$.str,$1.str,$2.str);
			}
		|	arrayel
		;	/* type str */

	/* everything but [,],l_IDENTIFIER,l_NEWLINE + idfplus */
arrayel		:	idfplus
			{ /* concatena le strighe */
			  char *elurl;
			  if ((elurl=hash_find(mainSTable,$1.idfstr.idf))!=NULL)
			    { /* link */
			      $$.str = (char *) malloc(strlen($1.idfstr.idf)+strlen($1.idfstr.succ)+
						       strlen(elurl)+18);
			      sprintf($$.str,"<A HREF=\"%s\">%s</A>%s",elurl,$1.idfstr.idf,$1.idfstr.succ);
			      free($1.idfstr.idf);
			      free($1.idfstr.succ);
			    }
			  else
			    {
			      strcatmem($$.str,$1.idfstr.idf,$1.idfstr.succ);
			    }
			}
		|	l_STRING
			{
			  strcat3mem($$.str,$1.strrec.pred,$1.strrec.mid,$1.strrec.succ);
			  free($1.strrec.name);
			}
		|	l_TPOINT
		|	l_OTHER
		|	l_TYPEDEF
		|	l_STRUNI
		|	l_TYPEIDF
		|	l_DEFINE
		|	l_INCLUDE
		|	l_PRECOMPILER
		|	'{'
		|	'}'
		|	'('
		|	')'
		|	'*'
		|	';'
		|	','
		|	'='
		;	/* type: str */

/* elements: include, define, functions, ... */

elements	:	elements element
			{ /* concatena le strighe */
			 strcatmem($$.str,$1.str,$2.str);
			}
		|	element
		;	/* type: str */

element		:	precompiler
		|	define
		|	include
		|	typedef
		|	function
		|	variable
		;	/* type: str */

/* precompiler */
precompiler	:	l_PRECOMPILER pcelses l_NEWLINE
			{ /* concatena le strighe +TAG*/
			 $$.str = (char *) malloc(strlen($1.str)+strlen($2.str)+
						strlen($3.str)+32);
			 strcpy($$.str,"<FONT COLOR=\"#A06000\">");
			 strcat($$.str,$1.str);
			 strcat($$.str,$2.str);
			 strcat($$.str,"</FONT>");
			 strcat($$.str,$3.str);

			 free($1.str);
			 free($2.str);
			 free($3.str);
			}
		;	/* type: str */

pcelse		:	pcelse nnlel
			{ /* concatena le due strighe */
			 strcatmem($$.str,$1.str,$2.str);
			}
		|	nnlel
		;	/* type: str */

pcelses		:	pcelse
		|	{ /* empty */
			 stremptymem($$.str);
			}
		;	/* type: str */

/* define */

define		:	define_name
		|	define_const
		|	define_func
		;	/* type: str */

define_name	:	l_DEFINE l_IDENTIFIER l_NEWLINE
			{ /* concatena le strighe +TAG*/
			 $$.str = (char *) malloc(strlen($1.str)+strlen($2.idfstr.idf)+
						strlen($2.idfstr.succ)+strlen($3.str)+32);
			 strcpy($$.str,"<FONT COLOR=\"#A06000\">");
			 strcat($$.str,$1.str);
			 strcat($$.str,$2.idfstr.idf);
			 strcat($$.str,$2.idfstr.succ);
			 strcat($$.str,"</FONT>");
			 strcat($$.str,$3.str);

			 free($1.str);
			 free($2.idfstr.idf);
			 free($2.idfstr.succ);
			 free($3.str);
			}
		;	/* type: str */

/* idf not followed by ( */
define_const	:	l_DEFINE l_IDENTIFIER dfnelse l_NEWLINE
			{ /* concatena le strighe +TAG*/
			 hash_add(mainSTable,$2.idfstr.idf,NULL);
			 hash_add(thisSTable,$2.idfstr.idf,NULL);

			 $$.str = (char *) malloc(strlen($1.str)+
						2*strlen($2.idfstr.idf)+strlen($2.idfstr.succ)+
						strlen($3.str)+strlen($4.str)+20);
			 strcpy($$.str,"<A NAME=\"");
			 strcat($$.str,$2.idfstr.idf);
			 strcat($$.str,"\">");
			 strcat($$.str,$1.str);
			 strcat($$.str,"<B>");
			 strcat($$.str,$2.idfstr.idf);
			 strcat($$.str,"</B>");
			 strcat($$.str,$2.idfstr.succ);
			 strcat($$.str,$3.str);
			 strcat($$.str,$4.str);
			 free($1.str);
			 free($2.idfstr.idf);
			 free($2.idfstr.succ);
			 free($3.str);
			 free($4.str);
			}
		;	/* type: str */



define_func	:	l_DEFINE l_FUNC dfnparams ')' dfnelse l_NEWLINE
			{ /* concatena le strighe +TAG*/
			 hash_add(mainSTable,$2.idfstr.idf,NULL);
			 hash_add(thisSTable,$2.idfstr.idf,NULL);

			 if (macrourl!=NULL)
			   {
			     $$.str = (char *) malloc(strlen($1.str)+strlen($2.idfstr.succ)+
						      3*strlen($2.idfstr.idf)+strlen(macrourl)+
						      strlen($3.str)+strlen($4.str)+
						      strlen($5.str)+strlen($6.str)+40);
			     
			     sprintf($$.str,"<A NAME=\"%s\">%s<B><A HREF=\"%s#%s\">%s</A></B>%s%s%s%s%s",
				     $2.idfstr.idf,$1.str,macrourl,$2.idfstr.idf,$2.idfstr.idf,
				     $2.idfstr.succ,$3.str,$4.str,$5.str,$6.str);
			   }
			 else
			   {
			     $$.str = (char *) malloc(strlen($1.str)+strlen($2.idfstr.succ)+
						      2*strlen($2.idfstr.idf)+
						      strlen($3.str)+strlen($4.str)+
						      strlen($5.str)+strlen($6.str)+20);
			     
			     sprintf($$.str,"<A NAME=\"%s\">%s<B>%s</B>%s%s%s%s%s",
				     $2.idfstr.idf,$1.str,$2.idfstr.idf,
				     $2.idfstr.succ,$3.str,$4.str,$5.str,$6.str);
			   }

			 free($1.str);
			 free($2.idfstr.idf);
			 free($2.idfstr.succ);
			 free($3.str);
			 free($4.str);
			 free($5.str);
			 free($6.str);
			}
		;	/* type: str */


dfnelse		:	dfnelse nnlel
			{ /* concatena le due strighe */
			 strcatmem($$.str,$1.str,$2.str);
			}
		|	nnlel
		;	/* type: str */

dfnparams	:	dfnparams ',' l_IDENTIFIER
			{ /* concatena le strighe */
			 strcat4mem($$.str,$1.str,$2.str,$3.idfstr.idf,$3.idfstr.succ);
			}
		|	l_IDENTIFIER
			{ /* concatena le strighe */
			 strcatmem($$.str,$1.idfstr.idf,$1.idfstr.succ);
			}
		;	/* type: str */

/* include */

include		:	l_INCLUDE l_STRING l_NEWLINE
			{ /* concatena le strighe +TAG*/
			 char *hashname;

			 if ((hashname=find_include($2.strrec.name))!=NULL)
			   {
			     char *htmlname;

			     htmlname = hash_load(mainSTable,hashname);

			     $$.str = (char *) malloc(strlen($1.str)+
						      strlen($2.strrec.pred)+strlen($2.strrec.mid)+strlen($2.strrec.succ)+
						      strlen($3.str)+strlen(htmlname)+64);
			     sprintf($$.str,"<FONT COLOR=\"#206020\">%s%s<A HREF=\"%s\">%s</A>%s</FONT>%s",
				     $1.str,$2.strrec.pred,htmlname,$2.strrec.mid,$2.strrec.succ,$3.str);

			     free(htmlname);
			     free(hashname);
			   }
			 else
			   {
			     $$.str = (char *) malloc(strlen($1.str)+
						      strlen($2.strrec.pred)+strlen($2.strrec.mid)+strlen($2.strrec.succ)+
						      strlen($3.str)+32);
			     strcpy($$.str,"<FONT COLOR=\"#206020\">");
			     strcat($$.str,$1.str);
			     strcat($$.str,$2.strrec.pred);
			     strcat($$.str,$2.strrec.mid);
			     strcat($$.str,$2.strrec.succ);
			     strcat($$.str,"</FONT>");
			     strcat($$.str,$3.str);
			   }


			 free($1.str);
			 free($2.strrec.pred);
			 free($2.strrec.mid);
			 free($2.strrec.succ);
			 free($2.strrec.name);
			 free($3.str);
			}
		;	/* type: str */

/* typedef */

typedef		:	l_TYPEDEF type idfplus ';'
			{ /* concatena le strighe +TAG*/
			 hash_add(mainSTable,$3.idfstr.idf,NULL);
			 hash_add(thisSTable,$3.idfstr.idf,NULL);

			 $$.str = (char *) malloc(strlen($1.str)+strlen($2.str)+
						2*strlen($3.idfstr.idf)+strlen($3.idfstr.succ)+
						strlen($4.str)+20);
			 strcpy($$.str,"<A NAME=\"");
			 strcat($$.str,$3.idfstr.idf);
			 strcat($$.str,"\">");
			 strcat($$.str,$1.str);
			 strcat($$.str,$2.str);
			 strcat($$.str,"<B>");
			 strcat($$.str,$3.idfstr.idf);
			 strcat($$.str,"</B>");
			 strcat($$.str,$3.idfstr.succ);
			 strcat($$.str,$4.str);

			 free($1.str);
			 free($2.str);
			 free($3.idfstr.idf);
			 free($3.idfstr.succ);
			 free($4.str);
			}
		;	/* type: str */

/* function */

function	:	type l_IDENTIFIER '(' fnparams ')' fnvars block
			{ /* concatena le strighe +TAG*/
			  /* the variables are necessary to support old type functions */
			 char *elurl;

			 if ((elurl=hash_find(mainSTable,$2.idfstr.idf))!=NULL)
			   { /* link */
			     $$.str = (char *) malloc(strlen($1.str)+2*strlen($2.idfstr.idf)+strlen($2.idfstr.succ)+
						      strlen($3.str)+strlen($4.str)+
						      strlen($5.str)+strlen($6.str)+strlen($7.str)+
						      strlen(elurl)+40);
			     sprintf($$.str,"<A NAME=\"%s\">%s<B><A HREF=\"%s\">%s</A></B>%s%s%s%s%s%s",
				     $2.idfstr.idf,$1.str,elurl,$2.idfstr.idf,$2.idfstr.succ,
				     $3.str,$4.str,$5.str,$6.str,$7.str);

			     /* delete previous definition */
			     hash_del(mainSTable,$2.idfstr.idf);
			     hash_del(thisSTable,$2.idfstr.idf);
			   }
			  else
			    {
			      if (funcurl==NULL)
				{
				  $$.str = (char *) malloc(strlen($1.str)+2*strlen($2.idfstr.idf)+strlen($2.idfstr.succ)+
							   strlen($3.str)+strlen($4.str)+
							   strlen($5.str)+strlen($6.str)+strlen($7.str)+20);
				  sprintf($$.str,"<A NAME=\"%s\">%s<B>%s</B>%s%s%s%s%s%s",
					  $2.idfstr.idf,$1.str,$2.idfstr.idf,$2.idfstr.succ,
					  $3.str,$4.str,$5.str,$6.str,$7.str);
				}
			      else
				{
				  $$.str = (char *) malloc(strlen($1.str)+3*strlen($2.idfstr.idf)+strlen($2.idfstr.succ)+
							   strlen($3.str)+strlen($4.str)+strlen(funcurl)+
							   strlen($5.str)+strlen($6.str)+strlen($7.str)+40);
				  sprintf($$.str,"<A NAME=\"%s\">%s<B><A HREF=\"%s#%s\">%s</A></B>%s%s%s%s%s%s",
					  $2.idfstr.idf,$1.str,funcurl,$2.idfstr.idf,$2.idfstr.idf,$2.idfstr.succ,
					  $3.str,$4.str,$5.str,$6.str,$7.str);
				}
			    }
			 
			 hash_add(mainSTable,$2.idfstr.idf,NULL);
			 hash_add(thisSTable,$2.idfstr.idf,NULL);

			 free($1.str);
			 free($2.idfstr.idf);
			 free($2.idfstr.succ);
			 free($3.str);
			 free($4.str);
			 free($5.str);
			 free($6.str);
			 free($7.str);
			}
		|	type l_IDENTIFIER '(' fnparams ')' ';'
			{ /* concatena le strighe +TAG*/
			 hash_add(mainSTable,$2.idfstr.idf,NULL);
			 hash_add(thisSTable,$2.idfstr.idf,NULL);

			 if (protourl!=NULL)
			   { /* link */
			     $$.str = (char *) malloc(strlen($1.str)+3*strlen($2.idfstr.idf)+strlen($2.idfstr.succ)+
						      strlen($3.str)+strlen($4.str)+
						      strlen($5.str)+strlen($6.str)+
						      strlen(protourl)+40);
			     sprintf($$.str,"<A NAME=\"%s\">%s<B><A HREF=\"%s#%s\">%s</A></B>%s%s%s%s%s",
				     $2.idfstr.idf,$1.str,protourl,$2.idfstr.idf,$2.idfstr.idf,$2.idfstr.succ,$3.str,$4.str,$5.str,$6.str);
			   }
			 else
			   {
			     $$.str = (char *) malloc(strlen($1.str)+2*strlen($2.idfstr.idf)+strlen($2.idfstr.succ)+
						      strlen($3.str)+strlen($4.str)+
						      strlen($5.str)+strlen($6.str)+20);
			     sprintf($$.str,"<A NAME=\"%s\">%s<B>%s</B>%s%s%s%s%s",
				     $2.idfstr.idf,$1.str,$2.idfstr.idf,$2.idfstr.succ,$3.str,$4.str,$5.str,$6.str);
			   }

			 free($1.str);
			 free($2.idfstr.idf);
			 free($2.idfstr.succ);
			 free($3.str);
			 free($4.str);
			 free($5.str);
			 free($6.str);
			}
		;	/* type: str */

fnparams	:	fnparams1
		|	{ /* empty */
			 stremptymem($$.str);
			}
		;	/* type: str */

fnparams1	:	fnparams1 ',' fnparam
			{ /* concatena le strighe */
			 strcat3mem($$.str,$1.str,$2.str,$3.str);
			}
		|	fnparam
		;	/* type: str */

fnparam		:	type idfplus
			{ /* concatena le strighe */
			 strcat3mem($$.str,$1.str,$2.idfstr.idf,$2.idfstr.succ);
			}
		|	type
		|	l_TPOINT
		|	type '(' '*' l_IDENTIFIER ')' '(' fnparams ')'
			{ /* concatena le strighe */
			 strcat9mem($$.str,$1.str,$2.str,$3.str,
				$4.idfstr.idf,$4.idfstr.succ,
				$5.str,$6.str,$7.str,$8.str);
			}
		;	/* type: str */

fnvars		:	fnvarsne
		|	{ /* empty */
			 stremptymem($$.str);
			}
		;	/* type: str */

fnvarsne	:	fnvarsne fnvar
			{ /* concatena le due strighe */
			 strcatmem($$.str,$1.str,$2.str);
			}
		|	fnvar
		;	/* type: str */

fnvar		:	type fnvaridfs ';'
			{ /* concatena le strighe*/
			 strcat3mem($$.str,$1.str,$2.str,$3.str);
			}
		|	type fnvaridfs '=' varelse ';'
			{ /* concatena le strighe*/
			 strcat5mem($$.str,$1.str,$2.str,$3.str,$4.str,$5.str);
			}
		;	/* type: str */

fnvaridfs	:	fnvaridfs ',' typevar idfplus
			{ /* concatena le strighe */
			 strcat5mem($$.str,$1.str,$2.str,$3.str,$4.idfstr.idf,$4.idfstr.succ);
			}
		|	idfplus
			{ /* concatena le strighe */
			 strcatmem($$.str,$1.idfstr.idf,$1.idfstr.succ);
			}
		;	/* type str */

block		:	'{' inblock '}'
			{ /* concatena le strighe */
			 strcat3mem($$.str,$1.str,$2.str,$3.str);
			}
		;	/* type: str */

inblock		:	inblock1
		|	{ /* empty */
			 stremptymem($$.str);
			}
		;	/* type: str */

inblock1	:	inblock1 inblockel
			{ /* concatena le strighe */
			 strcatmem($$.str,$1.str,$2.str);
			}
		|	inblockel
		;	/* type: str */

	/* everything but { or }*/
inblockel	:	l_IDENTIFIER
			{ /* concatena le strighe */
			  char *elurl;
			  if ((elurl=hash_find(mainSTable,$1.idfstr.idf))!=NULL)
			    { /* link */
			      $$.str = (char *) malloc(strlen($1.idfstr.idf)+strlen($1.idfstr.succ)+
						       strlen(elurl)+18);
			      sprintf($$.str,"<A HREF=\"%s\">%s</A>%s",elurl,$1.idfstr.idf,$1.idfstr.succ);
			      free($1.idfstr.idf);
			      free($1.idfstr.succ);
			    }
			  else
			    {
			      strcatmem($$.str,$1.idfstr.idf,$1.idfstr.succ);
			    }
			}
		|	l_FUNC
			{ /* concatena le strighe */
			  char *elurl;
			  if ((elurl=hash_find(mainSTable,$1.idfstr.idf))!=NULL)
			    { /* link */
			      $$.str = (char *) malloc(strlen($1.idfstr.idf)+strlen($1.idfstr.succ)+
						       strlen(elurl)+18);
			      sprintf($$.str,"<A HREF=\"%s\">%s</A>%s",elurl,$1.idfstr.idf,$1.idfstr.succ);
			      free($1.idfstr.idf);
			      free($1.idfstr.succ);
			    }
			  else
			    {
			      strcatmem($$.str,$1.idfstr.idf,$1.idfstr.succ);
			    }
			}
		|	l_STRING
			{
			  strcat3mem($$.str,$1.strrec.pred,$1.strrec.mid,$1.strrec.succ);
			  free($1.strrec.name);
			}
		|	l_TPOINT
		|	l_OTHER
		|	l_TYPEDEF
		|	l_STRUNI
		|	l_TYPEIDF
		|	l_NEWLINE
		|	l_DEFINE
		|	l_INCLUDE
		|	l_PRECOMPILER
		|	'('
		|	')'
		|	'['
		|	']'
		|	'*'
		|	';'
		|	','
		|	'='
		|	block
		;	/* type: str */

/* variable */

variable	:	type varidfs ';'
			{ /* concatena le strighe*/
			 strcat3mem($$.str,$1.str,$2.str,$3.str);
			}
		|	type varidfs '=' varelse ';'
			{ /* concatena le strighe*/
			 strcat5mem($$.str,$1.str,$2.str,$3.str,$4.str,$5.str);
			}
		;	/* type: str */

varidfs		:	varidfs ',' typevar idfplus
			{ /* concatena le strighe +TAG*/
			 hash_add(mainSTable,$4.idfstr.idf,NULL);
			 hash_add(thisSTable,$4.idfstr.idf,NULL);

			 $$.str = (char *) malloc(strlen($1.str)+strlen($2.str)+strlen($3.str)+
						2*strlen($4.idfstr.idf)+
						strlen($4.idfstr.succ)+20);
			 strcpy($$.str,$1.str);
			 strcat($$.str,$2.str);
			 strcat($$.str,$3.str);
			 strcat($$.str,"<A NAME=\"");
			 strcat($$.str,$4.idfstr.idf);
			 strcat($$.str,"\"><B>");
			 strcat($$.str,$4.idfstr.idf);
			 strcat($$.str,"</B>");
			 strcat($$.str,$4.idfstr.succ);

			 free($1.str);
			 free($2.str);
			 free($3.str);
			 free($4.idfstr.idf);
			 free($4.idfstr.succ);
			}
		|	idfplus
			{ /* concatena le strighe +TAG*/
			 hash_add(mainSTable,$1.idfstr.idf,NULL);
			 hash_add(thisSTable,$1.idfstr.idf,NULL);

			 $$.str = (char *) malloc(2*strlen($1.idfstr.idf)+
						strlen($1.idfstr.succ)+20);
			 strcpy($$.str,"<A NAME=\"");
			 strcat($$.str,$1.idfstr.idf);
			 strcat($$.str,"\"><B>");
			 strcat($$.str,$1.idfstr.idf);
			 strcat($$.str,"</B>");
			 strcat($$.str,$1.idfstr.succ);

			 free($1.idfstr.idf);
			 free($1.idfstr.succ);
			}
		;	/* type str */

varelse		:	varelse varelseel
			{ /* concatena le strighe */
			 strcatmem($$.str,$1.str,$2.str);
			}
		|	varelseel
		;	/* type: str */

	/* everything but ; */
varelseel	:	l_IDENTIFIER
			{ /* concatena le strighe */
			  char *elurl;
			  if ((elurl=hash_find(mainSTable,$1.idfstr.idf))!=NULL)
			    { /* link */
			      $$.str = (char *) malloc(strlen($1.idfstr.idf)+strlen($1.idfstr.succ)+
						       strlen(elurl)+18);
			      sprintf($$.str,"<A HREF=\"%s\">%s</A>%s",elurl,$1.idfstr.idf,$1.idfstr.succ);
			      free($1.idfstr.idf);
			      free($1.idfstr.succ);
			    }
			  else
			    {
			      strcatmem($$.str,$1.idfstr.idf,$1.idfstr.succ);
			    }
			}
		|	l_FUNC
			{ /* concatena le strighe */
			  char *elurl;
			  if ((elurl=hash_find(mainSTable,$1.idfstr.idf))!=NULL)
			    { /* link */
			      $$.str = (char *) malloc(strlen($1.idfstr.idf)+strlen($1.idfstr.succ)+
						       strlen(elurl)+18);
			      sprintf($$.str,"<A HREF=\"%s\">%s</A>%s",elurl,$1.idfstr.idf,$1.idfstr.succ);
			      free($1.idfstr.idf);
			      free($1.idfstr.succ);
			    }
			  else
			    {
			      strcatmem($$.str,$1.idfstr.idf,$1.idfstr.succ);
			    }
			}
		|	l_STRING
			{
			  strcat3mem($$.str,$1.strrec.pred,$1.strrec.mid,$1.strrec.succ);
			  free($1.strrec.name);
			}
		|	l_TPOINT
		|	l_OTHER
		|	l_TYPEDEF
		|	l_STRUNI
		|	l_TYPEIDF
		|	l_DEFINE
		|	l_INCLUDE
		|	l_PRECOMPILER
		|	'{'
		|	'}'
		|	l_NEWLINE
		|	')'
		|	'('
		|	'['
		|	']'
		|	'*'
		|	','
		|	'='
		;	/* type: str */

/* type */

type		:	typeidfs typenp stars
			{ /* concatena le strighe */
			 strcat3mem($$.str,$1.str,$2.str,$3.str);
			}
		|	typeidfs typenp
			{ /* concatena le strighe */
			 strcatmem($$.str,$1.str,$2.str);
			}
		;	/* type: str */

typenp		:	l_STRUNI block
			{ /* concatena le due strighe */
			 strcatmem($$.str,$1.str,$2.str);
			}
		|	l_STRUNI l_IDENTIFIER block
			{ /* concatena le strighe */
			 hash_add(mainSTable,$2.idfstr.idf,NULL);
			 hash_add(thisSTable,$2.idfstr.idf,NULL);

			 $$.str = (char *) malloc(strlen($1.str)+
						  2*strlen($2.idfstr.idf)+strlen($2.idfstr.succ)+
						  strlen($3.str)+24);
			 sprintf($$.str,"<A NAME=\"%s\">%s<B>%s</B>%s%s",$2.idfstr.idf,$1.str,$2.idfstr.idf,$2.idfstr.succ,$3.str);
			 free($1.str);
			 free($2.idfstr.idf);
			 free($2.idfstr.succ);
			 free($3.str);
			}
		|	l_STRUNI l_IDENTIFIER
			{ /* concatena le strighe */
			  char *elurl;
			  if ((elurl=hash_find(mainSTable,$2.idfstr.idf))!=NULL)
			    { /* link */
			      $$.str = (char *) malloc(strlen($1.str)+
						       strlen($2.idfstr.idf)+strlen($2.idfstr.succ)+
						       strlen(elurl)+18);
			      sprintf($$.str,"<A HREF=\"%s\">%s%s</A>%s",elurl,$1.str,$2.idfstr.idf,$2.idfstr.succ);
			      free($1.str);
			      free($2.idfstr.idf);
			      free($2.idfstr.succ);
			    }
			  else
			    {
			      strcat3mem($$.str,$1.str,$2.idfstr.idf,$2.idfstr.succ);
			    }
			}
		|	l_IDENTIFIER
			{ /* concatena le strighe */
			  char *elurl;
			  if ((elurl=hash_find(mainSTable,$1.idfstr.idf))!=NULL)
			    { /* link */
			      $$.str = (char *) malloc(strlen($1.idfstr.idf)+strlen($1.idfstr.succ)+
						       strlen(elurl)+18);
			      sprintf($$.str,"<A HREF=\"%s\">%s</A>%s",elurl,$1.idfstr.idf,$1.idfstr.succ);
			      free($1.idfstr.idf);
			      free($1.idfstr.succ);
			    }
			  else
			    {
			      strcatmem($$.str,$1.idfstr.idf,$1.idfstr.succ);
			    }
			}
		;	/* type: str */

typeidfs	:	typeidfs1
		|	{ /* empty */
			 stremptymem($$.str);
			}
		;	/* type: str */


typeidfs1	:	typeidfs1 l_TYPEIDF
			{ /* concatena le strighe */
			 strcatmem($$.str,$1.str,$2.str);
			}
		|	l_TYPEIDF
		;	/* type: str */

stars		:	stars '*'
			{ /* concatena le due strighe */
			 strcatmem($$.str,$1.str,$2.str);
			}
		|	'*'
		;	/* type: str */

	/* used if more than 1 variable is listed */
typevar		:	typeidfs stars
			{ /* concatena le due strighe */
			 strcatmem($$.str,$1.str,$2.str);
			}
		|	typeidfs
		;	/* type: str */


/************************************************************************/
%%

/* report yacc errors */
void yyerror(char *str)
{
 fprintf(stderr,"yacc line %7i: %s\n",Curr_Line,str);
}

#include "lex.yy.c"
